@ECHO OFF
REM ================================================================================
REM
REM Healthcare Pack for HIPAA X12
REM
REM Notices:
REM    LICENSED MATERIALS PROPERTY OF IBM AND HCL. 
REM       (c) Copyright International Business Machines Corp. 2006, 2017. 
REM       (c) Copyright HCL Technologies Ltd. 2017, 2019. All rights reserved. 
REM
REM    US Government Users Restricted Rights: 
REM       Use, duplication, or disclosure restricted by GSA ADP Schedule
REM       Contract with IBM Corp.  
REM		
REM ================================================================================
rem
rem USAGE: configurable_rules.bat  SEE ALSO: import_trees.bat
rem 
rem Intended to assist in customizing the hipaa_x12_qualifier.dat file
rem and the type trees used in the HIPAA compliance check application.
rem
rem For a complete description of how to configure HIPAA rules, refer to 
rem the Pack for healthcare documentation.
rem
rem This execution will process the configurable_rules_mini_qualifier.dat 
rem and update the on/off indicator in the main qualifier file 
rem (hipaa_x12_qualifier.dat) used in the compliance check application.
rem
rem The map source configurable_rules.mms contains executable maps
rem to complete this processing.
rem 
rem Please review all comments and instructions contained in this script 
rem and import_trees.bat
rem
REM ================================================================================
 
:BATCH_SETUP
	SETLOCAL
	SET this_script_name="%~nx0"
	TITLE Executing batch script file %this_script_name%
	SET dash=================================================================================

rem >>> This script will attempt to establish TX execution environment. Remove rem and set  
rem >>> TXHOME below ONLY if the script execution results cannot establish TX environment.
rem SET TXHOME=<TX product install path> 

REM Set debug_mode to NO to disable, DEBUG to enable
	SET debug_mode=NO
	IF /I [%1]==[DEBUG] SET debug_mode=DEBUG

	SET product_install_loc=unknown
	SET map_compile_exe=unknown
	SET cmd_server_exe=unknown
	
rem >>> Compile Executable Maps? (YES/NO)
rem >>> Import Trees? (YES/NO)
	SET CCMAPS=YES
	SET DOIMPORT=YES

:START_BATCH_PROCESSING
	PUSHD "%~dp0"
	IF %debug_mode%==DEBUG (
		ECHO Start location is %CD%
		PAUSE
	)
	
	CD ..\
	SET "configurable_rules_dir=%CD%"
	IF %debug_mode%==DEBUG (
		ECHO configurable_rules_dir is defined as %configurable_rules_dir%
		PAUSE
	)

REM cd to the health care install directory	
	CD ..\..\
	SET "healthcare_install_loc=%CD%"
	IF %debug_mode%==DEBUG (
		ECHO healthcare_install_loc is defined as %healthcare_install_loc%
		PAUSE
	)

:TX_ENV

    IF defined TXHOME goto SKIP_TX_ENV

REM get current directory name from path
    SET current_directory=%CD%
    for %%j in (%current_directory%) do set FOLDERNAME=%%~nj
    IF %debug_mode%==DEBUG (
		ECHO current directory name is %FOLDERNAME%
		PAUSE
    )

REM determine if HC pack install with ITXA or TX
    SET ITXA=FALSE
    IF %FOLDERNAME%==wtx (
		SET ITXA=TRUE
		IF %debug_mode%==DEBUG (
			ECHO ITXA execution is TRUE
			PAUSE
		)
    )

REM cd to the product install directory
    CD ..\..\
    IF %ITXA%==TRUE (
		IF defined DTXHOME SET TXHOME=%DTXHOME%
    ) ELSE (
       SET TXHOME=%CD%
    )
			
    IF defined TXHOME goto SKIP_TX_ENV

	SET SPE_INSTALL_DIR=%CD%
    IF not exist "%SPE_INSTALL_DIR%\bin\spewtxenv.bat" goto :ERROR_ON_TX_ENVIRONMENT

REM call ITXA script to set TX environment
    IF %debug_mode%==DEBUG (
		ECHO call %SPE_INSTALL_DIR%\bin\spewtxenv.bat
		PAUSE
    )

	SET InSPEConsole=1
    call "%SPE_INSTALL_DIR%\bin\spewtxenv.bat"
	
    SET TXHOME=%DTXHOME%
    IF defined TXHOME (
	   goto SKIP_TX_ENV
    ) ELSE (
	   goto :ERROR_ON_TX_ENVIRONMENT
	)
	
:SKIP_TX_ENV 
		
	SET "product_install_loc=%TXHOME%"
	CD %product_install_loc%
    IF %debug_mode%==DEBUG (
		ECHO Product install location is %product_install_loc%
		PAUSE
	)

REM check to make sure mcompile and cmdsvr are where we expect them
	
	IF EXIST mcompile.exe (
		SET "map_compile_exe=%product_install_loc%\mcompile.exe"
		IF EXIST dtxcmdsv64.exe (
			SET "cmd_server_exe=%product_install_loc%\dtxcmdsv64.exe"
		) ELSE (
			IF EXIST dtxcmdsv.exe (
				SET "cmd_server_exe=%product_install_loc%\dtxcmdsv.exe"
			) ELSE (
				GOTO :PRODUCT_NOT_FOUND
			)
		)
	) ELSE (	
		GOTO :ERROR_PRODUCT_NOT_FOUND
	)
	
	CD %configurable_rules_dir%

	IF %debug_mode%==DEBUG (
		ECHO New location is %CD%
		ECHO map_compile_exe is defined as %map_compile_exe%
		ECHO cmd_server_exe is defined as %cmd_server_exe%
		PAUSE
	)

:COMPILE_MAPS
	IF "%CCMAPS%"=="NO" GOTO :RUN_CONFIGURABLE_RULES

	"%map_compile_exe%" "%configurable_rules_dir%\maps\configurable_rules.mms" -A -K
	IF %ERRORLEVEL% NEQ 0 GOTO :ERROR_ON_COMPILE_MAPS
	
	IF EXIST "%configurable_rules_dir%\maps\*.mme" (
		DEL "%configurable_rules_dir%\maps\*.mme"
	)
	IF EXIST "%configurable_rules_dir%\maps\*.mdg" (
		DEL "%configurable_rules_dir%\maps\*.mdg"
	)

:RUN_CONFIGURABLE_RULES
REM ================================================================================
rem
rem EXECUTABLE MAP: configurable_rules.mmc
rem
rem This is the main execution map for the process.  The inputs and outputs
rem are distinct based on product install (ITXA or TX).  File locations relative
rem to %configurable_rules_dir% variable within this script.
rem
rem Detail file descriptions are located at the end of this file.
rem
REM ================================================================================
rem
rem ITXA INPUT: 
rem      Card 1 - ..\..\data\hipaa_x12_qualifier.dat
rem      Card 2 - ..\trees\hipaa_x12_master_rule_stub.mts
rem	     Card 3 - ..\data\HIPAA_Business_Rules_Export_from_SPE.xml
rem ITXA OUTPUT:
rem      Card 1 - ..\data\configurable_rules_results.out
rem      Card 2 - ..\data\hipaa_x12_qualifier_updated.dat
rem      Card 3 - ..\data\merged_mini_and_component_rules.out
rem      Card 4 - ..\data\qualifier_report.html
rem      Card 5 - ..\trees\hipaa_x12_master_rule_updated.mts
rem          
REM ================================================================================
rem
rem TX INPUT:
rem      Card 1 - ..\data\configurable_rules_mini_qualifier.dat 
rem      Card 2 - ..\..\data\hipaa_x12_qualifier.dat
rem      Card 3 - ..\data\configurable_rules_component_rules_only.dat
rem      Card 4 - ..\trees\hipaa_x12_master_rule_stub.mts
rem      Card 5 - ..\master\directory_paths.dat
rem      Card 6 - ..\data\configurable_rules_type_5_mini_qualifier.dat 
rem TX OUTPUT:
rem      Card 1 - ..\data\configurable_rules_results.out
rem      Card 2 - ..\data\hipaa_x12_qualifier_updated.dat
rem      Card 3 - ..\data\merged_mini_and_component_rules.out
rem      Card 4 - ..\data\qualifier_report.html
rem      Card 5 - ..\trees\hipaa_x12_master_rule_updated.mts
rem
REM ================================================================================          

	ECHO %dash%
	ECHO:
	ECHO                      HIPAA X12 RULES CONFIGURATION
	ECHO:
	ECHO %dash%
	@ECHO OFF

	"%cmd_server_exe%" '%configurable_rules_dir%\maps\configurable_rules.mmc' -B 
	IF %debug_mode%==DEBUG (
		ECHO %configurable_rules_dir%\maps\configurable_rules.mmc   return code is %ERRORLEVEL%
		PAUSE
	)
	IF %ERRORLEVEL% NEQ 0 GOTO :ERROR_ON_RUN_CONFIGURABLE_RULES

	ECHO %dash%
	ECHO:
	ECHO	MAP EXECUTION SUCCESSFUL configurable_rules.mmc
	ECHO:
	ECHO %dash%
	@ECHO OFF

:IMPORT_TREES
	IF "%DOIMPORT%"=="NO" GOTO :WARNING_FOR_NO_IMPORT_TREES

	IF %debug_mode%==DEBUG (
		ECHO call "%configurable_rules_dir%\master\import_trees.bat"
		PAUSE
	)
	
	IF %debug_mode%==DEBUG (
		CALL "%configurable_rules_dir%\master\import_trees.bat" DEBUG
	) ELSE (
		CALL "%configurable_rules_dir%\master\import_trees.bat"
	)
	IF %ERRORLEVEL% NEQ 0 GOTO :ERROR_ON_IMPORT_TREES

rem 
rem >>> END OF PROCESSING 
rem

	ECHO %dash%
	ECHO:
	ECHO  FINAL RESULTS:  SUCCESS  New Compliance Check Files  
	ECHO:
	ECHO     %configurable_rules_dir%
	ECHO:
	ECHO         ..\data\hipaa_x12_qualifier_updated.dat
	ECHO:
	ECHO         ..\trees\hipaa_x12_type_2.mtt
	ECHO         ..\trees\hipaa_x12.mtt
	ECHO         ..\trees\hipaa_x12_ruleless.mtt
	ECHO:
	ECHO %dash%

	GOTO :END

rem
rem >>> ERROR AND WARNING MESSAGES
rem

:ERROR_ON_TX_ENVIRONMENT
	ECHO %dash%
	ECHO:
	ECHO: RESULTS:  CANNOT ESTABLISH TX ENVIRONMENT 
	ECHO: FILE NOT FOUND %SPE_INSTALL_DIR%\bin\spewtxenv.bat 
	ECHO:
	ECHO %dash%
	@ECHO OFF
	GOTO :END

:ERROR_ON_COMPILE_MAPS
	ECHO %dash%
	ECHO:
	ECHO: RESULTS:  ERROR  MAP COMPILE ERROR 
	ECHO:
	ECHO %dash%
	@ECHO OFF
	GOTO :END

:ERROR_ON_RUN_CONFIGURABLE_RULES
	ECHO %dash%
	ECHO:
	ECHO: RESULTS:  ERROR  MAP EXECUTION FAILED configurable_rules.mmc
	ECHO:
	ECHO %dash%
	@ECHO OFF
	GOTO :END

:ERROR_ON_IMPORT_TREES
	ECHO %dash%
	ECHO:
	ECHO: RESULTS:  ERROR  import_trees.bat FAILED
	ECHO:
	ECHO %dash%
	@ECHO OFF
	GOTO :END

:WARNING_FOR_NO_IMPORT_TREES
	ECHO %dash%
	ECHO:
	ECHO  RESULTS:  PARTIAL  New Compliance Check Files  
	ECHO:
	ECHO     %configurable_rules_dir%
	ECHO:
	ECHO         ..\data\hipaa_x12_qualifier_updated.dat
	ECHO:
	ECHO: WARNING: import_trees.bat WAS NOT EXECUTED
	ECHO:          Trees and qualifier rules are not in sync 
	ECHO:
	ECHO %dash%
	@ECHO OFF
	GOTO :END
	
:ERROR_PRODUCT_NOT_FOUND
	ECHO %dash%
	ECHO:
	ECHO Key product executables were not found! 
	IF %map_compile_exe%==unknown ECHO     mcompile.exe was not found
	IF %cmd_server_exe%==unknown ECHO     dtxcmdsv[64].exe was not found
	ECHO Searching in the relative product install location 
	ECHO     %product_install_loc%
	ECHO:
	ECHO %dash%
	@ECHO OFF
	IF %debug_mode%==DEBUG PAUSE
	GOTO :END

:END
	POPD
	IF %debug_mode%==DEBUG PAUSE

	IF %ERRORLEVEL% NEQ 0 (
		ENDLOCAL
		EXIT /B 1
	) 
	
	ENDLOCAL
	EXIT /B
REM ================================================================================
rem INPUT FILE DESCRIPTIONS:
rem
rem    ..\data\configurable_rules_mini_qualifier.dat (customized rules)
rem
rem          A mini qualifier file that contains the type 3 and 4 rules from the
rem          hipaa_x12_qualifier.dat) file including an on/off indicator.  
rem             ITXA Customers:  This file is generated using exported rules found
rem                in file HIPAA_Business_Rules_Export_from_SPE.xml.
rem             TX Customers:  Will need to manually update this file.   
rem          The customized qualifier file will be validated and a report will be
rem          produced identifying any errors, modified, unchanged, and enabled
rem          non-configured rules. 
rem
rem    ..\..\data\hipaa_x12_qualifier.dat
rem
rem          The main qualifier file used in the compliance check application.  
rem          NOTE: File Locations
rem                ITXA install_dir\Healthcare_Payer_vn.n.n\wtx\hipaa\data 
rem                TX install_dir\packs\healthcare_vn.n.n.n\hipaa\data
rem
rem    ..\data\configurable_rules_component_rules_only.dat
rem
rem          ITXA Customers:  This file is not used.
rem          TX Customers:  Contains tree component rules for each qualifier rule.
rem
rem    ..\trees\hipaa_x12_master_rule_stub.mts
rem
rem          Contains the master tree components
rem
rem    ..\master\directory_paths.dat
rem
rem          ITXA Customers:  This file is not used.
rem          TX Customers:  Contains file paths for file locations. 
rem             Path 1 identified the path to the master directory, 
rem             Path 2 beginning with (*) identifies path to the tree directory.
rem
rem    ..\data\HIPAA_Business_Rules_Export_from_SPE.xml
rem
rem          ITXA Customers:  Contains customized rules exported from ITXA
rem             to be used to create ..\data\configurable_rules_mini_qualifier.dat
rem
rem    ..\data\configurable_rules_type_5_mini_qualifier.dat (customized rules)
rem  
rem          ITXA Customers:  This file is not used.
rem          TX Customers:  A mini qualifier file that will contains the type 5
rem             rules from the (hipaa_x12_qualifier.dat) file including an on/off 
rem             indicator. Customers will need to update this file manually.  The
rem             customized qualifier file will be validated and a report will be
rem             produced identifying any errors, modified, unchanged, and enabled
rem             non-configured rules. 
REM ================================================================================
rem OUTPUT FILE DESCRIPTIONS:
rem
rem    ..\data\configurable_rules_results.out
rem
rem          Contains a processing trace.
rem
rem    ..\data\hipaa_x12_qualifier_updated.dat
rem
rem          ITXA Customers:  This file is not used. But should be retained for
rem             product support purposes.
rem          TX Customers:  Contains the updated rule for the main qualifier file 
rem             (hipaa_x12_qualifier.dat).  This file should be copied to the
rem             compliance check application location:
rem
rem                ITXA install_dir\Healthcare_Payer_vn.n.n\wtx\hipaa\data 
rem                TX install_dir\packs\healthcare_vn.n.n.n\hipaa\data
rem
rem          NOTE:  The original qualifier file installed with the HIPAA pack
rem          should be copied to a backup location before using  this file.
rem
rem    ..\data\merged_mini_and_component_rules.out
rem 
rem          Contains the tree component rules to be added/removed in the HIPAA
rem          compliance check type trees.  
rem
rem    ..\data\qualifier_report.html
rem
rem          Contains an HTML report identifying any errors, modified, unchanged,
rem          and enabled non-configured rules.  
rem
rem    ..\trees\hipaa_x12_master_rule_updated.mts
rem
rem          The updated master tree containing all HIPAA component rules.
REM ================================================================================